/*
 *      wireless_calls.c  gwireless_applet
 *
 *      Copyright 2000 Sujal Shah
 *
 * This file contains the main code for dealing with the 
 * wireless subsystem in Linux (for now).  Many details in
 * this code have been adapted from the iwconfig source 
 * code.
 *
 * Code is released under the GNU GPL.  See http://www.fsf.org.
 *
 */

#include "wireless_calls.h"

int parse_procwireless(InterfaceInfo * indata)
{

	InterfaceInfo * wdata = indata;

	static FILE * pf = NULL;

	int i = 0;

	char buff[255];
	char status[5];
	char * procbits;

	int tempint=0;

	if (pf == NULL)
	{
		/* open /proc/net/wireless */
		pf = fopen ("/proc/net/wireless", "r");

		if (pf == NULL)
		{
				static int warned = 0;
				if (!warned)
				{
						printf("Cannot open /proc/net/wireless.  Is wireless support built into the kernel\n");
						warned = 1;
				}

				wdata->stats.qual.qual=0;
				wdata->stats.qual.level=0;
				wdata->stats.qual.noise=0;
				return -1;
		}
	} else {
		/* /proc/net/wireless is already open */
		int fd_copy;
				
		fd_copy = dup(fileno(pf));
		if (fd_copy == -1)
		{
			printf("Could not dup file handle: %d", errno);
			exit(1);
		}

		fclose(pf);
		pf = fdopen(fd_copy, "r");
		fseek(pf, 0, SEEK_SET);
				
	}

	for (i = 0; i < 2; i++)
	{
			/*char speedbuff; */
			int trash;
			while ( (trash = getc(pf)) != '\n' && (trash != EOF))
			{
			}
	}

	/* only supports first interface right now 
	* this is temporary
	*/
	if (fgets (buff, sizeof(buff) -1, pf) != NULL)
	{
		buff[sizeof(buff)-1] = '\0';

		procbits = buff;

		sscanf(buff, "%6s:",
					wdata->name);

		procbits += 7;

		procbits = strtok(procbits, " ");
		sscanf(procbits, "%4s", status);
		/*procbits = strtok(NULL, " ");
		sscanf(procbits, "%4s", status);
		*/
		procbits = strtok(NULL, " ");
		sscanf(procbits, "%d.", &tempint);
		wdata->stats.qual.qual = (unsigned char) tempint;
		procbits = strtok(NULL, " ");
		sscanf(procbits, "%d.", &tempint);
		wdata->stats.qual.level = (unsigned char) tempint;
		procbits = strtok(NULL, " ");
		sscanf(procbits, "%d.", &tempint);
		wdata->stats.qual.noise = (unsigned char) tempint;
		procbits = strtok(NULL, " ");
		sscanf(procbits, "%d.", &wdata->stats.discard.nwid);
		procbits = strtok(NULL, " ");
		sscanf(procbits, "%d.", &wdata->stats.discard.code);
		procbits = strtok(NULL, " ");
		sscanf(procbits, "%d.", &wdata->stats.discard.misc);
	
#ifdef DEBUG
		printf ("%s %d, %d, %d, %d, %d, %d\n", wdata->name, wdata->stats.qual.qual, 
						wdata->stats.qual.level, wdata->stats.qual.noise, wdata->stats.discard.nwid,
					wdata->stats.discard.code, wdata->stats.discard.misc);
#endif

	} else {
			return -1;
	}

	return 0;


}

int grab_iw_info(InterfaceInfo * indata)
{

	InterfaceInfo * wdata = indata;
	/*struct iw_range rangedata;*/
	struct iwreq request;


	/* open socket */

	int netsock_fd = -1;

	netsock_fd = socket(AF_INET, SOCK_DGRAM,0);

	if (netsock_fd == -1)
			return -1;

	/* network name */

	strcpy ( request.ifr_name, wdata->name);
	request.u.essid.pointer = (caddr_t) wdata->essid;
	request.u.essid.length=0;
	request.u.essid.flags=0;
	if ( ioctl (netsock_fd, SIOCGIWESSID, &request) >= 0)
	{
			wdata->has_essid = 1;
			wdata->essid_on = request.u.data.flags;

	}

	/* station name */

	strcpy ( request.ifr_name, wdata->name);
	request.u.essid.pointer = (caddr_t) wdata->nickname;
	request.u.essid.length=0;
	request.u.essid.flags=0;
	if ( ioctl (netsock_fd, SIOCGIWNICKN, &request) < 0)
	{
			wdata->nickname[0] = '\0';
	}

	/* mode */

	strcpy ( request.ifr_name, wdata->name);
	if ( ioctl (netsock_fd, SIOCGIWMODE, &request) >= 0)
	{
			if ((request.u.mode < 6) && (request.u.mode >= 0))
					wdata->mode = request.u.mode;
	}

	/* enc settings (on/off and key) */
	wdata->has_key = 0;
	strcpy ( request.ifr_name, wdata->name);
	request.u.data.pointer = (caddr_t) wdata->key;
	request.u.data.length=0;
	request.u.data.flags=0;
#ifdef DEBUG
	printf ("immediate before SIOCGIWENCODE\n");
#endif
	if ( ioctl (netsock_fd, SIOCGIWENCODE, &request) >= 0)
	{
#ifdef DEBUG
			printf ("ioctl SIOCGIWENCODE returned positive or zero\n");
#endif
			wdata->has_key = 1;
			wdata->key_size = request.u.data.length;
			wdata->key_flags = request.u.data.flags;
	}

	/* power savings */
	strcpy(request.ifr_name, wdata->name);
  	request.u.power.flags = 0;
  	if(ioctl(netsock_fd, SIOCGIWPOWER, &request) >= 0)
    	{
      		wdata->has_power = 1;
      		memcpy(&(wdata->power), &(request.u.power), sizeof(iwparam));
    	}

	/* access point mac address */
	strcpy(request.ifr_name, wdata->name);
	if(ioctl(netsock_fd, SIOCGIWAP, &request) >= 0)
	{
		wdata->has_ap_addr = 1;
		memcpy(&(wdata->ap_addr), &(request.u.ap_addr), sizeof (sockaddr));
	}

	/* pack it up and spit it out */

	close (netsock_fd);

	return 0;
}

int determine_max_values(InterfaceInfo* indata)
{

	struct iw_range rangedata;
	/*struct iw_param * params;*/
	struct iwreq request;
	char  buffer [ sizeof (struct iw_range) * 2 ];
	int netsock_fd = -1;
	InterfaceInfo * wdata = indata;
	netsock_fd = socket(AF_INET, SOCK_DGRAM,0);

	if (netsock_fd == -1)
			return -1;

	/* the iwconfig source says this is big enuogh, but
	 * why is that?
	 */

	memset (buffer, 0, sizeof(rangedata));

	strcpy(request.ifr_name, wdata->name);
	request.u.data.pointer = buffer;
	request.u.data.length = 0;
	request.u.data.flags = 0;
	if (ioctl(netsock_fd, SIOCGIWRANGE, &request) < 0)
		return (-1);

	memcpy((char *) &rangedata, buffer, sizeof(struct iw_range));

#ifdef DEBUG
	printf ("%d, %d, %d\n",rangedata.max_qual.qual, 
						rangedata.max_qual.level,
						rangedata.max_qual.noise);
#endif

	close(netsock_fd);


	return 0;

}

int set_iw_info(InterfaceInfo * indata)
{

	InterfaceInfo * wdata = indata;
	/*struct iw_range rangedata;*/
	struct iwreq request;

	/* open socket */
	int netsock_fd = -1;

	netsock_fd = socket(AF_INET, SOCK_DGRAM,0);

	if (netsock_fd == -1)
			return -1;

	strncpy(request.ifr_name, indata->name, IFNAMSIZ);

	if (indata->has_essid)
	{
			if (strlen(indata->essid) > IW_ESSID_MAX_SIZE)
			{
					fprintf(stderr, "ESSID too long (max %d): ``%s''\n",
									IW_ESSID_MAX_SIZE, indata->essid);
			} else {
				if((!strcasecmp(indata->essid, "off")) ||
					(!strcasecmp(indata->essid, "any")))
				{
					request.u.essid.flags = 0;
					indata->essid[0] = '\0';
					request.u.essid.length=1;
				} else {
					request.u.essid.flags = 1;
					request.u.essid.pointer = (caddr_t) indata->essid;
					request.u.essid.length = strlen(indata->essid) + 1;
				}
					if(ioctl(netsock_fd, SIOCSIWESSID, &request) < 0)
					{
						fprintf(stderr, "SIOCSIWESSID: %s\n", strerror(errno));
						return(-1);
					}
			}
	}

	if (indata->has_nickname)
	{
		strncpy(request.ifr_name, indata->name, IFNAMSIZ);

		if(strlen(indata->nickname) > IW_ESSID_MAX_SIZE)
		{
			fprintf(stderr, "SIOCSIWNICKN: Name too long (max %d) : ``%s''\n",
					IW_ESSID_MAX_SIZE, indata->nickname);
		}

		request.u.essid.pointer = (caddr_t) indata->nickname;
		request.u.essid.length = strlen(indata->nickname) + 1;
		if(ioctl(netsock_fd, SIOCSIWNICKN, &request) < 0)
		{
			fprintf(stderr, "SIOCSIWNICKN: %s\n", strerror(errno));
			return(-1);
		}
	}

	if (indata->has_mode)
	{
		strncpy(request.ifr_name, indata->name, IFNAMSIZ);

		request.u.mode = indata->mode;
		if(ioctl(netsock_fd, SIOCSIWMODE, &request) < 0)
		{
			fprintf(stderr, "SIOCSIWMODE: %s\n", strerror(errno));
			return(-1);
		}
	}

	if (indata->has_key)
	{
		strncpy(request.ifr_name, indata->name, IFNAMSIZ);
		request.u.data.flags = indata->key_flags;

		if (strlen(indata->key) <=0)
		{
			request.u.data.pointer = NULL;
		} else {
			request.u.data.pointer = indata->key;
		}

		request.u.data.length = indata->key_size;

		if(ioctl(netsock_fd, SIOCSIWENCODE, &request) < 0)
		{
			fprintf(stderr, "SIOCSIWENCODE(%d): %s\n",
				errno, strerror(errno));
			return(-1);
		}

	}

	if (indata->has_power)
	{
		strncpy(request.ifr_name, indata->name, IFNAMSIZ);

		/* if power off */
		if (indata->power.disabled)
		{
				request.u.power.disabled = 1;
		} else {
			/* Get old Power info */
			if (ioctl (netsock_fd, SIOCGIWPOWER, &request) < 0)
			{
				fprintf (stderr, "SIOCGIWPOWER: %s\n", strerror (errno));
				return (-1);
			}
			strcpy (request.ifr_name, indata->name);
			request.u.power.disabled = 0;
		}

		if (ioctl (netsock_fd, SIOCSIWPOWER, &request) < 0)
		{
			fprintf (stderr, "SIOCSIWPOWER(%d): %s\n",
					errno, strerror (errno));
			return (-1);
		}
	}

	close(netsock_fd);
}

void initialize_iwinfo(InterfaceInfo * indata)
{

  	indata->has_nwid=0;
  	indata->has_freq=0;
  	indata->has_sens=0;
  	indata->has_key=0;
  	indata->has_essid=0;
  	indata->has_nickname=0;
  	indata->has_ap_addr=0;
  	indata->has_bitrate=0;
  	indata->has_rts=0;
  	indata->has_frag=0;
  	indata->has_mode=0;
  	indata->has_power=0;


}
