/*
 * gwireless.h
 *
 * This file contains data structures and function declarations
 * used by the gwireless applet.
 *
 * Copyright 2001 Sujal Shah
 *
 * This file is released under the GNU GPL.  See
 * http://www.fsf.org for more information and for
 * a copy of the license.
 *
 */

#ifndef _GWIRELESS_APPLET_H_
#define _GWIRELESS_APPLET_H_

#include <fcntl.h>
#include <errno.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <applet-widget.h>
#include "callbacks.h"
#include "support.h"
#include <wireless_calls.h>

#define MAX_LINK_QUALITY 40
#define MAX_NOISE 90

typedef struct wireless_applet
{

	GtkWidget *applet;
	GtkWidget *main_box;
	GtkWidget *signalwidget;

	gchar * ifname;
	gchar essid[IW_ESSID_MAX_SIZE+1];
	gchar * status;
	gchar nickname[IW_ESSID_MAX_SIZE+1];
	unsigned char key [IW_ENCODING_TOKEN_MAX];
	gint link_quality;
	gint level;
	gint noise;
	gint mode;
	gint key_size;
	gint key_flags;

	int essid_on;
	int has_essid;
	int has_key;
	int has_mode;

	InterfaceInfo iw_info;

	/* runtime data */
	gint update_interval;
	guint update_to_id;

	/* panel layout/orientation stuff */
	PanelOrientType orient;
	gint size;

	/* used by event handlers to determine whether to
	* force the screen update (needed by refresh handlers, etc.
	*/
	gboolean force;

	GdkColor block_color;

	/* detail setting stuff */
	GtkWidget *DetailSetDialog;
	GtkWidget *netname_fld;
	GtkWidget *nickname_fld;
	GtkWidget *managed_rb;
	GtkWidget *adhoc_rb;
	GtkWidget *key_fld;
	GtkWidget *encswitch;
	GtkWidget *power_toggle;

	GdkGC *myGC;


} WirelessApplet;

void showDetails(AppletWidget *applet, gpointer data);
gint initialize_display(AppletWidget *applet);
gint signal_update(gpointer data);


#endif
